package main

import (
	"compress/gzip"
	"errors"
	"io"
	"net/http"
	"strings"
)

type gzCloser struct {
	r     *gzip.Reader
	under io.ReadCloser
}

func (gz *gzCloser) Read(p []byte) (int, error) {
	return gz.r.Read(p)
}

func (gz *gzCloser) Close() error {
	defer gz.under.Close()
	return gz.r.Close()
}

func httpget(url string) (*http.Response, error) {
	client := &http.Client{}
	req, err := http.NewRequest("GET", url, nil)
	if err != nil {
		return nil, err
	}
	req.Header.Set("User-Agent", "Mozilla/5.0 (X11; CrOS x86_64 11021.56.0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/70.0.3538.76 Safari/537.36")
	req.Header.Set("Accept", "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8")
	req.Header.Set("Accept-Language", "en-US,en;q=0.9")
	req.Header.Set("Accept-Encoding", "gzip")
	resp, err := client.Do(req)
	if err != nil {
		return nil, err
	}
	if resp.StatusCode != 200 {
		resp.Body.Close()
		return nil, errors.New("not 200")
	}
	if strings.EqualFold(resp.Header.Get("Content-Encoding"), "gzip") {
		gz, err := gzip.NewReader(resp.Body)
		if err != nil {
			resp.Body.Close()
			return nil, err
		}
		resp.Body = &gzCloser{r: gz, under: resp.Body}
	}
	return resp, nil
}
