package main

import (
	"fmt"
	"html/template"
	"log"
	"net/url"
	"regexp"
	"strconv"
	"strings"
	"time"

	"github.com/andybalholm/cascadia"
	"golang.org/x/net/html"
)

type Tweet struct {
	Author string
	Source string
	Images []template.HTML
	Text   template.HTML
	Quote  template.HTML
}

var tweetview = template.Must(template.ParseFiles("views/tweet.html"))
var authorregex = regexp.MustCompile("twitter.com/([^/]+)")
var tweetsel = cascadia.MustCompile("p.tweet-text")
var quotsel = cascadia.MustCompile(".QuoteTweet-authorAndText")
var linksel = cascadia.MustCompile(".time a.tweet-timestamp")
var imgsel = cascadia.MustCompile(".AdaptiveMedia-photoContainer")
var timestampsel = cascadia.MustCompile(".time span._timestamp")

func proctweets(feed *Feed, baseurl *url.URL, root *html.Node) {
	divs := tweetsel.MatchAll(root)
	authormatch := authorregex.FindStringSubmatch(feed.Feedurl)
	source := authormatch[1]
	now := time.Now()
	for _, div := range divs {
		twp := div.Parent.Parent.Parent
		tweet := &Tweet{}

		alink := linksel.MatchFirst(twp)
		link := "https://twitter.com" + getattr(alink, "href")
		if havelink(feed.Feedid, link) {
			continue
		}

		tweet.Source = source
		authormatch = authorregex.FindStringSubmatch(link)
		tweet.Author = authormatch[1]
		tweet.Text = cleannode(div, baseurl)
		if quot := quotsel.MatchFirst(twp); quot != nil {
			tweet.Quote = cleannode(quot, baseurl)
		}
		for _, imgdiv := range imgsel.MatchAll(twp) {
			tweet.Images = append(tweet.Images,
				template.HTML(createimgloader(baseurl,
					getattr(imgdiv, "data-image-url"), "", "")))
		}

		dt := &now
		if timespan := timestampsel.MatchFirst(twp); timespan != nil {
			datatime := getattr(timespan, "data-time")
			secs, err := strconv.ParseInt(datatime, 10, 0)
			if err == nil {
				ts := time.Unix(secs, 0)
				dt = &ts
			}
		}

		title := fmt.Sprintf("Tweet from %s", tweet.Author)

		var content strings.Builder
		err := tweetview.ExecuteTemplate(&content, "tweet.html", tweet)
		if err != nil {
			log.Printf("template error: %s", err)
			continue
		}

		savelink(feed.Feedid, link, title, dt, content.String())
	}
}

func loadtweets(feed *Feed, baseurl *url.URL) {
	resp, err := httpget(feed.Feedurl)
	if err != nil {
		log.Printf("error loading %s: %s", feed.Feedurl, err)
		return
	}
	root, err := html.Parse(resp.Body)
	if err != nil {
		log.Printf("error parsing %s: %s", feed.Feedurl, err)
		return
	}
	proctweets(feed, baseurl, root)
}
